// -----------------------------------------------------------------------
// MI.30xx series Win 31 Software driver example (c) Spectrum GmbH 07/2001
// -----------------------------------------------------------------------
//
// This example will only work correct, if just one MI.30xx is installed
// in the system and no other Spectrum boards.
// Not all software registers are used, see the hardware manual for
// additional registers
//
// -----------------------------------------------------------------------
//
// This example works with all MI.30xx boards. You may delete the lines
// that are not necessary for your type of board
//

// -----------------------------------------------------------------------
//
// Be carefully when using interrupt mode: You need a second task to stop
// recording if no trigger event occurs or you need to terminate the task
// with the task manager,
//
// -----------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>



// ----- driver includes -----
#include "dlltyp.h"
#include "spectrum.h"
#include "errors.h"
#include "regs.h"



// ----- main task -----
int main()
	{
	int16	nCount, nPCIBusVersion, nChannels;
	int32	lBrdType, lInstMemsize, lSerialNumber;
	int32	lChEnable, lStatus, lMemsize;
	int32	lErrorCode, lErrorReg, lErrorValue;
	ptr16	pnData[4];
	ptr16	pnTmp;
	int32	i;
	int16	j, nErr;


	// ----- Initialisation of PCI Bus ---------------------------------------
	if (SpcInitPCIBoards (&nCount, &nPCIBusVersion) != ERR_OK) 
		return -1;
	if (nCount == 0) 
		{
		printf ("No Spectrum board found\n"); 
		return -1;
		}

	// ----- request and print Board type and some information -----
	SpcGetParam (0, SPC_PCITYP,			&lBrdType);
	SpcGetParam (0, SPC_PCIMEMSIZE,		&lInstMemsize);
	SpcGetParam (0, SPC_PCISERIALNO,	&lSerialNumber);
	printf ("Board found:     MI.%x\n", lBrdType);
	printf ("Memory on board: %ld MBytes (%ld MSamples)\n", lInstMemsize / 1024 / 1024, lInstMemsize / 1024 / 1024 /2);
	printf ("Serial Number:   %05ld\n", lSerialNumber);

	// ----- Record maximum number of channels available -----
	switch (lBrdType)
		{
		case TYP_MI3010:
		case TYP_MI3020: nChannels = 1; lChEnable = CHANNEL0; break;

		case TYP_MI3011:
		case TYP_MI3012:
		case TYP_MI3015:
		case TYP_MI3021:
		case TYP_MI3022:
		case TYP_MI3025:
		case TYP_MI3031: nChannels = 2; lChEnable = CHANNEL0 | CHANNEL1; break;
		
		case TYP_MI3013:
		case TYP_MI3014:
		case TYP_MI3016:
		case TYP_MI3023:
		case TYP_MI3024:
		case TYP_MI3026:
		case TYP_MI3033: nChannels = 4; lChEnable = CHANNEL0 | CHANNEL1 | CHANNEL2 | CHANNEL3; break;
		}

	// ----- set memsize for recording ----
	lMemsize = 1024;

	// ----- setup board for recording -----
	for (j=0; j<nChannels; j++)
		SpcSetParam (0, SPC_AMP0 + 100*j,	1000);			// +/- 1V input range
	for (j=0; j<nChannels; j++)
		SpcSetParam (0, SPC_50OHM0 + 100*j,	1);				// 50 Ohm input impedance

	SpcSetParam (0, SPC_CHENABLE,			lChEnable);		// Enable channels for recording
	SpcSetParam (0, SPC_POSTTRIGGER,		lMemsize/2);	// Trigger event in middle of data
	SpcSetParam (0, SPC_MEMSIZE,			lMemsize);		// Memory size

	SpcSetParam (0, SPC_PLL_ENABLE,			1);				// Internal PLL enabled for clock
	SpcSetParam (0, SPC_EXTERNALCLOCK,		0);				// Internal clock used
	SpcSetParam (0, SPC_SAMPLERATE,			10000000l);		// Samplerate: 10 MHz
	SpcSetParam (0, SPC_EXTERNOUT,			0);				// No clock output
	
	SpcSetParam (0, SPC_TRIGGERMODE,		TM_SOFTWARE);	// Software trigger is used
	SpcSetParam (0, SPC_PULSEWIDTH,			0);				// Not used for software mode
	SpcSetParam (0, SPC_TRIGGEROUT,			0);				// No trigger output


	// ----- start the board -----
	nErr = SpcSetParam (0, SPC_COMMAND,		SPC_START);		// Use SPC_STARTANDWAIT for interrupt mode

	// ----- driver error: request error and end program -----
	if (nErr != ERR_OK)
		{
		SpcGetParam (0, SPC_LASTERRORCODE,	&lErrorCode);
		SpcGetParam (0, SPC_LASTERRORREG,	&lErrorReg);
		SpcGetParam (0, SPC_LASTERRORVALUE,	&lErrorValue);

		printf ("Driver error: %ld in register %ld at value %ld\n", lErrorCode, lErrorReg, lErrorValue);

		return -1;
		}

	// ----- Wait for Status Ready (Could be deleted if interrupt mode is used) -----
	do
		{
		SpcGetParam (0, SPC_STATUS,		&lStatus);		
		}
	while (lStatus != SPC_READY);

	printf ("Board is ready, recording has finished\n");

	// ----- allocate memory for data -----
	for (i=0; i<nChannels; i++)
		pnData[i] = (ptr16) malloc (lMemsize * sizeof(int16));

	// ----- data position is depending on type of board, see manual for details -----
	switch (lBrdType)
		{

		// ----- one channel recorded -----
		case TYP_MI3010:
		case TYP_MI3020:
			SpcGetData (0, 0, 0, lMemsize, (dataptr) pnData[0]);
			break;

		// ----- two channels recorded on memory channel 0 -----
		case TYP_MI3011:
		case TYP_MI3012:
		case TYP_MI3021:
		case TYP_MI3022:
		case TYP_MI3031:
			pnTmp = (ptr16) malloc (lMemsize * 2 * sizeof(int16));	// allocate temporary buffer for copy
			SpcGetData (0, 0, 0, 2*lMemsize, (dataptr) pnTmp);		// get both channels together
			for (i=0; i<lMemsize; i++)								// split data in the two channels
				{
				pnData[0][i] = pnTmp[2*i];
				pnData[1][i] = pnTmp[2*i+1];
				}
			free (pnTmp);											// free temporary buffer
			break;

		// ----- two channels recorded on both memory channels -----
		case TYP_MI3015:
		case TYP_MI3025:
			SpcGetData (0, 0, 0, lMemsize, (dataptr) pnData[0]);
			SpcGetData (0, 1, 0, lMemsize, (dataptr) pnData[1]);
			break;

		// ----- four channels recorded -----
		case TYP_MI3013:
		case TYP_MI3014:
		case TYP_MI3016:
		case TYP_MI3023:
		case TYP_MI3024:
		case TYP_MI3026:
		case TYP_MI3033: 
			pnTmp = (ptr16) malloc (lMemsize * 2 * sizeof(int16));	// allocate temporary buffer for copy
			for (j=0; j<2; j++)										// two memory channels to process
				{
				SpcGetData (0, j, 0, 2*lMemsize, (dataptr) pnTmp);	// get both channels together
				for (i=0; i<lMemsize; i++)							// split data in the two channels
					{
					pnData[2*j+0][i] = pnTmp[2*i];
					pnData[2*j+1][i] = pnTmp[2*i+1];
					}
				}
			free (pnTmp);											// free temporary buffer
			break;
		}

	// ----- output first 20 samples in hex format -----
	printf ("Data:\n\n");
	for (i=0; i<20; i++)
		{
		for (j=0; j<nChannels; j++)
			printf ("%4x  ", pnData[j][i] & 0xFFF);
		printf ("\n");
		}

	// ----- free data memory -----
	for (i=0; i<nChannels; i++)
		free (pnData[i]);

	return 0;
	}
