// -----------------------------------------------------------------------
// MI.31xx series Win 31 Software driver example (c) Spectrum GmbH 09/2001
// -----------------------------------------------------------------------
//
// This example will only work correct, if just one MI.31xx is installed
// in the system and no other Spectrum boards.
// Not all software registers are used, see the hardware manual for 
// additional registers
//
// -----------------------------------------------------------------------
//
// This example works with all MI.31xx boards. You may delete the lines
// that are not necessary for your type of board
//

// -----------------------------------------------------------------------
//
// Be carefully when using interrupt mode: You need a second task to stop
// recording if no trigger event occurs or you need to terminate the task
// with the task manager,
//
// -----------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

// ----- driver includes -----
#include "dlltyp.h"
#include "spectrum.h"
#include "errors.h"
#include "regs.h"



// ----- main task -----
int main()
	{
	int16	nCount, nPCIBusVersion, nChannels, nChPerMod, nMod;
	int32	lBrdType, lInstMemsize, lSerialNumber;
	int32	lChEnable, lStatus, lMemsize;
	int32	lErrorCode, lErrorReg, lErrorValue;
	ptr16	pnData[8];
	ptr16	pnTmp;
	int32	i, j, k;
	int16	nErr;


	// ----- Initialisation of PCI Bus ---------------------------------------
	if (SpcInitPCIBoards (&nCount, &nPCIBusVersion) != ERR_OK)
		return -1;
	if (nCount == 0)
		{
		printf ("No Spectrum board found\n"); 
		return -1;
		}

	// ----- request and print Board type and some information -----
	SpcGetParam (0, SPC_PCITYP,			&lBrdType);
	SpcGetParam (0, SPC_PCIMEMSIZE,		&lInstMemsize);
	SpcGetParam (0, SPC_PCISERIALNO,	&lSerialNumber);
	printf ("Board found:     MI.%x\n", lBrdType);
	printf ("Memory on board: %ld MBytes (%ld MSamples)\n", lInstMemsize / 1024 / 1024, lInstMemsize / 1024 / 1024 /2);
	printf ("Serial Number:   %05ld\n", lSerialNumber);

	// ----- Record maximum number of channels available -----
	switch (lBrdType)
		{
		case TYP_MI3110:
		case TYP_MI3120: 
		case TYP_MI3130: 
			nChannels = 2; 
			nChPerMod = 2;
			nMod = 1;
			lChEnable = CHANNEL0 | CHANNEL1; 
			break;

		case TYP_MI3111:
		case TYP_MI3121: 
		case TYP_MI3131: 
			nChannels = 4; 
			nChPerMod = 4;
			nMod = 1;
			lChEnable = CHANNEL0 | CHANNEL1 | CHANNEL2 | CHANNEL3;
			break;

		case TYP_MI3112:
		case TYP_MI3122: 
		case TYP_MI3132: 
			nChannels = 8; 
			nChPerMod = 4;
			nMod = 1;
			lChEnable = CHANNEL0 | CHANNEL1 | CHANNEL2 | CHANNEL3 | CHANNEL4 | CHANNEL5 | CHANNEL6 | CHANNEL7; 
			break;
		}

	// ----- set memsize for recording ----
	lMemsize = 1024;

	// ----- setup board for recording -----
	for (j=0; j<nChannels; j++)
		SpcSetParam (0, SPC_AMP0 + 100*j,	1000);			// +/- 1V input range
	for (j=0; j<nChannels; j++)
		SpcSetParam (0, SPC_50OHM0 + 100*j,	1);				// 50 Ohm input impedance

	SpcSetParam (0, SPC_CHENABLE,			lChEnable);		// Enable channels for recording
	SpcSetParam (0, SPC_POSTTRIGGER,		lMemsize/2);	// Trigger event in middle of data
	SpcSetParam (0, SPC_MEMSIZE,			lMemsize);		// Memory size

	SpcSetParam (0, SPC_PLL_ENABLE,			1);				// Internal PLL enabled for clock
	SpcSetParam (0, SPC_EXTERNALCLOCK,		0);				// Internal clock used
	SpcSetParam (0, SPC_SAMPLERATE,			1000000l);		// Samplerate: 1 MHz
	SpcSetParam (0, SPC_EXTERNOUT,			0);				// No clock output
	SpcSetParam (0, SPC_TRIGGERMODE,		TM_SOFTWARE);	// Software trigger is used
	SpcSetParam (0, SPC_PULSEWIDTH,			0);				// Not used for software mode
	SpcSetParam (0, SPC_TRIGGEROUT,			0);				// No trigger output


	// ----- start the board -----
	nErr = SpcSetParam (0, SPC_COMMAND,		SPC_START);		// Use SPC_STARTANDWAIT for interrupt mode

	// ----- driver error: request error and end program -----
	if (nErr != ERR_OK)
		{
		SpcGetParam (0, SPC_LASTERRORCODE,	&lErrorCode);
		SpcGetParam (0, SPC_LASTERRORREG,	&lErrorReg);
		SpcGetParam (0, SPC_LASTERRORVALUE,	&lErrorValue);

		printf ("Driver error: %ld in register %ld at value %ld\n", lErrorCode, lErrorReg, lErrorValue);

		return -1;
		}

	// ----- Wait for Status Ready (Could be deleted if interrupt mode is used) -----
	do
		{
		SpcGetParam (0, SPC_STATUS,		&lStatus);		
		}
	while (lStatus != SPC_READY);

	printf ("Board is ready, recording has finished\n");

	// ----- allocate memory for data -----
	for (i=0; i<nChannels; i++)
		pnData[i] = (ptr16) malloc (lMemsize * sizeof(int16));

	// ----- data is muxed and must be splitted -----
	pnTmp = (ptr16) malloc (lMemsize * nChPerMod * sizeof(int16));
	for (j=0; j < nMod; j++)
		{
		SpcGetData (0, j, 0, nChPerMod * lMemsize, (dataptr) pnTmp);
		for (i=0; i<lMemsize; i++)
			for (k=0; k<nChPerMod; k++)
				pnData[nChPerMod * j + k][i] = pnTmp[nChPerMod * i + k];
		}
	free (pnTmp);

	// ----- output first 20 samples in hex format -----
	printf ("Data:\n\n");
	for (i=0; i<20; i++)
		{
		for (j=0; j<nChannels; j++)
			printf ("%4x  ", pnData[j][i] & 0xFFF);
		printf ("\n");
		}

	// ----- free data memory -----
	for (i=0; i<nChannels; i++)
		free (pnData[i]);

	return 0;
	}
